---------------------------------------------------------------------
-- XML-RPC over HTTP.
-- See Copyright Notice in license.html
-- $Id: http.lua,v 1.2 2004/09/27 16:39:37 tomas Exp $
---------------------------------------------------------------------

require"socket.http"
require"ltn12"
require"xmlrpc"

xmlrpc.http = {}

---------------------------------------------------------------------
-- Call a remote method.
-- @param url String with the location of the server.
-- @param method String with the name of the method to be called.
-- @return Table with the response (could be a `fault' or a `params'
--	XML-RPC element).
---------------------------------------------------------------------
function xmlrpc.http.call (url, method, ...)
	local body = {}
	local data = xmlrpc.clEncode (method, unpack (arg))
	local rc, code, headers = socket.http.request {
		method = "POST",
		source = ltn12.source.string(data),
		url = url,
		headers = {
			["User-agent"] = "LuaXMLRPC",
			["Content-type"] = "text/xml",
			["Content-length"] = tostring(string.len(data)),
		},
		sink = ltn12.sink.table(body)
	}
	body = table.concat(body)
	if rc == 1 and tonumber (code) == 200 then
		return xmlrpc.clDecode (body)
	else
		error (code)
	end
end
